@@ -6,13 +6,13 @@ function error {
 }
 
 modconf=""
-tmp=`mktemp -q /tmp/nvidia.XXXXXX` || error "mktemp"
+mktemp=`which mktemp || echo touch`
+tmp=`$mktemp -q /tmp/nvidia.XXXXXX` || error "mktemp"
 
-paths[0]=/etc/conf.modules
-paths[1]=/etc/modules.conf
-paths[2]=/etc/modutils/aliases
+paths="/etc/conf.modules /etc/modules.conf"
+paths="$path /etc/modutils/aliases"
 
-for path in ${paths[@]}; do
+for path in $paths; do
   test -f $path && modconf=$path
 done
 
@@ -30,6 +30,7 @@ else
     sed '/.*NVdriver.*/d' < /etc/rc.d/rc.modules > $tmp
     echo "/sbin/modprobe NVdriver" >> $tmp
     mv $tmp /etc/rc.d/rc.modules
+    chmod 755 /etc/rc.d/rc.modules
   fi
 fi
 
