@@ -57,8 +57,10 @@
 #include <asm/uaccess.h>		// access_ok
 #include <asm/io.h>
 #include <asm/softirq.h>                // mark_bh
+
 #if !defined (KERNEL_2_2)
 #include <linux/spinlock.h>
+#include <linux/highmem.h>
 #else
 #include <asm/spinlock.h>
 #endif
@@ -871,7 +873,7 @@ static int nv_map_device(nv_state_t *nv)
 
     nv->fb.map   = NULL;
     nv->fb.map_u = NULL;
-
+    
     nv->regs.map   = NULL;
     nv->regs.map_u = NULL;
     
@@ -1585,6 +1587,7 @@ int nv_kern_ioctl(
     unsigned int cmd,
     unsigned long i_arg)
 {
+    int size;
     int status = 0;
     nv_linux_state_t *nvl;
     nv_state_t *nv;
@@ -1595,11 +1598,13 @@ int nv_kern_ioctl(
 
     NV_DMSG(nv, "ioctl(0x%x, 0x%x)", _IOC_NR(cmd), (unsigned int) i_arg);
 
-    switch (_IOC_NR(cmd))
+    size = _IOC_SIZE(cmd);
+
+    switch (cmd)
     {
         /* debug tool; zap the module use count so we can unload driver */
         /*             even if it is confused */
-        case _IOC_NR(NV_IOCTL_MODULE_RESET):
+        case NV_IOCTL_MODULE_RESET:
             atomic_set(&__this_module.uc.usecount, 1);
             break;
 
@@ -1607,7 +1612,7 @@ int nv_kern_ioctl(
            used by X driver to make card quiet while Xserver unnecessarily
            diddles around in its PCI config space
          */
-        case _IOC_NR(NV_IOCTL_INTERRUPTS):
+        case NV_IOCTL_INTERRUPTS:
         {
             int state = i_arg;
             ACTUAL_DEVICE_ONLY(nv);
@@ -1619,7 +1624,7 @@ int nv_kern_ioctl(
             break;
         }
 
-        case _IOC_NR(NV_IOCTL_MEMORY_VTOP):
+        case NV_IOCTL_MEMORY_VTOP:
         {
             nv_ioctl_memory_vtop_t params;
             nv_ioctl_memory_vtop_t *userbuf = arg;
@@ -1635,7 +1640,7 @@ int nv_kern_ioctl(
         }
 
         /* pass out info about the card */
-        case _IOC_NR(NV_IOCTL_CARD_INFO):
+        case NV_IOCTL_CARD_INFO:
         {
             nv_ioctl_card_info_t ci;
             nv_ioctl_card_info_t *userbuf = arg;
@@ -1669,7 +1674,7 @@ int nv_kern_ioctl(
         }
 
 	/* set a card to be primary (not post it) */
-	case _IOC_NR(NV_IOCTL_PRIMARY_CARD):
+	case NV_IOCTL_PRIMARY_CARD:
 	{
 	    nv_ioctl_primary_card_t params;
 	    nv_ioctl_primary_card_t *userbuf = arg;
@@ -1685,7 +1690,7 @@ int nv_kern_ioctl(
 	}
 
         default:
-            status = nvExtEscape(file, nv, _IOC_NR(cmd), arg) ? 0 : -EINVAL;
+            status = nvExtEscape(file, nv, cmd, arg) ? 0 : -EINVAL;
             break;
     }
 
@@ -2185,7 +2190,7 @@ int nv_vtop(
     l = NV_ALIGN_UP((unsigned long)params->byte_length +
                     ((unsigned long)params->buffer - (unsigned long)v), NVHW_PAGE_SIZE);
 
-    for (physical_addresses = (u32 *) (NV_UINTPTR_T) params->physical_addresses;
+    for (physical_addresses = params->physical_addresses;
          l > 0;
          physical_addresses++)
     {
@@ -2234,7 +2239,7 @@ nv_get_phys_address(unsigned long address)
 {
     pgd_t *pg_dir;
     pmd_t *pg_mid_dir;
-    pte_t *pg_table;
+    pte_t *pte__, pte;
 
     /* XXX do we really need this? */
     if (address > VMALLOC_START)
@@ -2262,11 +2267,19 @@ nv_get_phys_address(unsigned long address)
     if (pmd_none(*pg_mid_dir))
         goto failed;
 
-    pg_table = pte_offset(pg_mid_dir, address);
-    if (!pte_present(*pg_table))
+#if defined (pte_offset_atomic)
+    pte__ = pte_offset_atomic(pg_mid_dir, address);
+    pte = *pte__;
+    pte_kunmap(pte__);
+#else
+    pte__ = NULL;
+    pte = *pte_offset(pg_mid_dir, address);
+#endif
+
+    if (!pte_present(pte)) 
         goto failed;
 
-    return ((pte_val(*pg_table) & KERN_PAGE_MASK) | NV_MASK_OFFSET(address));
+    return ((pte_val(pte) & KERN_PAGE_MASK) | NV_MASK_OFFSET(address));
 
   failed:
     return (unsigned long) NULL;
@@ -2541,13 +2554,14 @@ void nv_lock_rm(
 #if defined(NV_DEBUG_LOCKS)
     int me = (in_interrupt() ? (in_irq() ? -1 : -2) : current->pid);
     int me_waiting = 0;
+    int cpu = smp_processor_id();
 
     // make sure we want to debug locks..
     if (locking_debuglevel && nvl->rm_locked) // &&
         // if we're not debugging ISRs, make sure we're not in an interrupt
         // ((locking_debuglevel < DEBUG_ISR_LOCKING) ? !in_interrupt() : TRUE))
     {
-        nv_printf("(%d): %d waiting on rm (%d)\n", current->processor, me,
+        nv_printf("(%d): %d waiting on rm (%d)\n", cpu, me,
             nvl->rm_locked);
         if ((nvl->rm_locked == current->pid) && (!in_interrupt()))
         {
@@ -2573,7 +2587,7 @@ void nv_lock_rm(
         if (locking_debuglevel > DEBUG_DEADLOCKS)
         {
             if ((locking_debuglevel < DEBUG_ISR_LOCKING) ? !in_interrupt() : TRUE)
-                nv_printf("(%d): %d rm\n", current->processor, nvl->rm_locked);
+                nv_printf("(%d): %d rm\n", cpu, nvl->rm_locked);
         }
         else
         {
@@ -2591,14 +2605,19 @@ void nv_unlock_rm(
     nv_state_t *nv
 )
 {
-    nv_linux_state_t *nvl = NV_GET_NVL_FROM_NV_STATE(nv);
+    int cpu;
+    nv_linux_state_t *nvl;
+
+    cpu = smp_processor_id();
+    nvl = NV_GET_NVL_FROM_NV_STATE(nv);
+
 #if defined(NV_DEBUG_LOCKS)
     if (locking_debuglevel)
     {
         if (nvl->rm_waiting ||
             ((locking_debuglevel > DEBUG_DEADLOCKS) &&
              ((locking_debuglevel < DEBUG_ISR_LOCKING) ? !in_interrupt() : TRUE)))
-                nv_printf("(%d): %d drop rm\n", current->processor, nvl->rm_locked);
+                nv_printf("(%d): %d drop rm\n", cpu, nvl->rm_locked);
     }
     nvl->rm_locked = 0;
 #endif
@@ -2614,13 +2633,14 @@ static void nv_lock_ldata(
 #if defined(NV_DEBUG_LOCKS)
     int me = (in_interrupt() ? (in_irq() ? -1 : -2) : current->pid);
     int me_waiting = 0;
+    int cpu = smp_processor_id();
 
     // make sure we want to debug locks..
     if (locking_debuglevel && nvl->ldata_locked) // &&
         // if we're not debugging ISRs, make sure we're not in an interrupt
         // ((locking_debuglevel < DEBUG_ISR_LOCKING) ? !in_interrupt() : TRUE))
     {
-        nv_printf("(%d): %d wait on ldata (%d)\n", current->processor, me,
+        nv_printf("(%d): %d wait on ldata (%d)\n", cpu, me,
             nvl->ldata_locked);
         if (nvl->ldata_locked == current->pid)
             nv_printf("  I already own the lock!\n");
@@ -2643,7 +2663,7 @@ static void nv_lock_ldata(
         if (locking_debuglevel > DEBUG_DEADLOCKS)
         {
             if ((locking_debuglevel < DEBUG_ISR_LOCKING) ? !in_interrupt() : TRUE)
-                nv_printf("(%d): %d ldata\n", current->processor, nvl->ldata_locked);
+                nv_printf("(%d): %d ldata\n", cpu, nvl->ldata_locked);
         }
         else
             if (me_waiting)
@@ -2659,7 +2679,11 @@ static void nv_unlock_ldata(
     nv_state_t *nv
 )
 {
-    nv_linux_state_t *nvl = NV_GET_NVL_FROM_NV_STATE(nv);
+    int cpu;
+    nv_linux_state_t *nvl;
+        
+    cpu = smp_processor_id();
+    nvl = NV_GET_NVL_FROM_NV_STATE(nv);
 
 #if defined(NV_DEBUG_LOCKS)
     if (locking_debuglevel)
@@ -2667,7 +2691,7 @@ static void nv_unlock_ldata(
         if (nvl->ldata_waiting ||
             ((locking_debuglevel > DEBUG_DEADLOCKS) &&
              ((locking_debuglevel < DEBUG_ISR_LOCKING) ? !in_interrupt() : TRUE)))
-                nv_printf("(%d): %d drop ldata\n", current->processor, nvl->ldata_locked);
+                nv_printf("(%d): %d drop ldata\n", cpu, nvl->ldata_locked);
     }
     nvl->ldata_locked = 0;
 #endif
@@ -2685,13 +2709,14 @@ static void nv_lock_at(
 #if defined(NV_DEBUG_LOCKS)
     int me = (in_interrupt() ? (in_irq() ? -1 : -2) : current->pid);
     int me_waiting = 0;
+    int cpu = smp_processor_id();
 
     // make sure we want to debug locks..
     if (locking_debuglevel && nvl->at_locked) // &&
         // if we're not debugging ISRs, make sure we're not in an interrupt
         // ((locking_debuglevel < DEBUG_ISR_LOCKING) ? !in_interrupt() : TRUE))
     {
-        nv_printf("(%d): %d wait on at (%d)\n", current->processor, me,
+        nv_printf("(%d): %d wait on at (%d)\n", cpu, me,
             nvl->at_locked);
         if (nvl->at_locked == current->pid)
             nv_printf("  I already own the lock!\n");
@@ -2714,7 +2739,7 @@ static void nv_lock_at(
         if (locking_debuglevel > DEBUG_DEADLOCKS)
         {
             if ((locking_debuglevel < DEBUG_ISR_LOCKING) ? !in_interrupt() : TRUE)
-                nv_printf("(%d): %d at\n", current->processor, nvl->at_locked);
+                nv_printf("(%d): %d at\n", cpu, nvl->at_locked);
         }
         else
             if (me_waiting)
@@ -2730,7 +2755,11 @@ static void nv_unlock_at(
     nv_state_t *nv
 )
 {
-    nv_linux_state_t *nvl = NV_GET_NVL_FROM_NV_STATE(nv);
+    int cpu;
+    nv_linux_state_t *nvl;
+        
+    cpu = smp_processor_id();
+    nvl = NV_GET_NVL_FROM_NV_STATE(nv);
 
 #if defined(NV_DEBUG_LOCKS)
     if (locking_debuglevel)
@@ -2738,7 +2767,7 @@ static void nv_unlock_at(
         if (nvl->at_waiting ||
             ((locking_debuglevel > DEBUG_DEADLOCKS) &&
              ((locking_debuglevel < DEBUG_ISR_LOCKING) ? !in_interrupt() : TRUE)))
-                nv_printf("(%d): %d drop at\n", current->processor, nvl->at_locked);
+                nv_printf("(%d): %d drop at\n", cpu, nvl->at_locked);
     }
     nvl->at_locked = 0;
 #endif
@@ -2756,10 +2785,11 @@ void nv_lock_bh(
 #if defined(NV_DEBUG_LOCKS)
     int me = (in_irq() ? -1 : -2 );
     int bh_waiting = 0;
+    int cpu = smp_processor_id();
 
     if (locking_debuglevel && nvl->bh_locked)
     {
-        nv_printf("(%d) %d wait on bh\n", current->processor, me);
+        nv_printf("(%d) %d wait on bh\n", cpu, me);
         if (nvl->bh_locked == -1)
             nv_printf("  ISR owns the lock!!\n");
         if (nvl->bh_locked == -2)
@@ -2776,7 +2806,7 @@ void nv_lock_bh(
     if (locking_debuglevel)
     {
         if (locking_debuglevel > DEBUG_DEADLOCKS)
-            nv_printf("(%d): %d bh\n", current->processor, nvl->bh_locked);
+            nv_printf("(%d): %d bh\n", cpu, nvl->bh_locked);
         else
             if (bh_waiting)
                 nv_printf("   Got it!!\n");
@@ -2788,11 +2818,15 @@ void nv_unlock_bh(
     nv_state_t *nv
 )
 {
-    nv_linux_state_t *nvl = NV_GET_NVL_FROM_NV_STATE(nv);
+    int cpu;
+    nv_linux_state_t *nvl;
+    
+    cpu = smp_processor_id();
+    nvl = NV_GET_NVL_FROM_NV_STATE(nv);
 
 #if defined(NV_DEBUG_LOCKS)
     if (locking_debuglevel >= DEBUG_ISR_LOCKING)
-        nv_printf("(%d): %d drop bh\n", current->processor, nvl->bh_locked);
+        nv_printf("(%d): %d drop bh\n", cpu, nvl->bh_locked);
     nvl->bh_locked = 0;
 #endif
 
