@@ -1092,6 +1092,15 @@ typedef struct _def_hw_info
         U032 CH7009_03h;                    // default positioning value for reg 3
         U032 CH7009_05h;                    // default positioning value for reg 5
         U032 CH7009_06h;                    // default positioning value for reg 6
+        U032 CH7009_07h;                    // current BLR/brightness setting (w/o Macrovision compensation)
+        S032 CH7009_07hAdjust;              // BLR/brightness adjustment (based on Macrovision)
+        U032 CH700X_08h;                    // default positioning value for reg 8
+        U032 CH700X_0Ah;                    // default positioning value for reg A
+        U032 CH700X_0Bh;                    // default positioning value for reg B
+        U032 PH710X_70h;                    // Default hpos values for Philips encoders                     
+        U032 PH710X_71h;                    
+        U032 PH710X_72h;
+        U032 PH710X_VDT;
         NV_CFGEX_TV_USER_CONTROLS_PARAMS TVControls; // TV slider controls
         U032 CursorColor1;
         U032 CursorColor2;
@@ -1141,7 +1150,6 @@ typedef struct _def_hw_info
         U032 HorizontalSyncPolarity;
         U032 VerticalSyncPolarity;
         U032 DoubleScannedMode;
-        U032 fpPixClk;
         U032 fpHActive;
         U032 fpHDispEnd;
         U032 fpHValidEnd;
@@ -1168,6 +1176,8 @@ typedef struct _def_hw_info
         DACHALINFO HalInfo;
         DACDCBDEVDESC DCBDesc[DCB_MAX_NUM_DEVDESCS];
         DACDCBI2CRECORD DCBI2c[DCB_MAX_NUM_I2C_RECORDS];
+        U032 VBiosDCB[DCB_MAX_NUM_DEVDESCS][2];
+        U032 VBiosICB[DCB_MAX_NUM_DEVDESCS];
 #define DAC_CRT_OFFSET_IN_MAP       0       // bits 0..7 map to CRT0..CRT7
 #define DAC_TV_OFFSET_IN_MAP        8       // bits 8..15 map to TV0..TV7
 #define DAC_DD_OFFSET_IN_MAP        16      // bits 16..23 map to DD0..DD7
@@ -1184,6 +1194,7 @@ typedef struct _def_hw_info
             U032 RMMaintainDevs;            //Ignore VBIOS input and do it ourselves.
             U032 RMMaintainScaling;         //Ignore VBIOS input and do it ourselves.
             U032 LidBehaviorBitMask;
+            U032 MaskSecondaryDevice;       //which devices shouldnt be enabled as secondary by default
 #define NV_MOBILE_LID_HIDES_DEV         1   //If set, lid closed causes us to consider LCD not present
 #define NV_MOBILE_LID_HOTKEYS_NOSPAN    2   //If set, lid closed with LCD enabled causes spoofed hotkey (standard and clone mode)
 #define NV_MOBILE_LID_HOTKEYS_SPAN      4   //If set, lid closed with LCD enabled causes spoofed hotkey (spanning mode)
@@ -1203,6 +1214,7 @@ typedef struct _def_hw_info
                                     // panel in on or off state
             U032 DevSwitchSuppressMask; //Conditions for which to suppress display switching.
                                         // Bits defines in nvcm.h with NV_CFG_DISPLAY_CHANGE_INHIBITS.
+            U032 SBIOSHandlesHotkeyInhibit; // Non-zero if the SBIOS can be trusted to inhibit hotkeys correctly.
         } DeviceProperties;
         U032 DevicesNoCheck;
 #define NO_DEVICES_FORCED_PRESENT   0
@@ -1310,6 +1322,9 @@ typedef struct _def_hw_info
         U032 TVFlickerFilter;
         U032 TVSaturation;
 
+        //these are not saved off on a per-head basis, because they don't vary from head to head (yet!)
+        CURRENT_TV_POSITION_ADJUSTMENT  TVCurrentPosition;
+
 #define TVO_CONTRAST      BIT(0)
 #define TVO_BRIGHTNESS    BIT(1)
 #define TVO_FLICKERFILTER BIT(2)
@@ -1319,6 +1334,9 @@ typedef struct _def_hw_info
         U032 TVInitFromRegistry;
         U032 dynEngCtrlEnabled;
         U032 unusedEngines;
+        U032 modesetBlankDelay;
+        U032 enableDvoPm;
+        U032 DCBOverrideRegistry;
     } Dac;
 
 #ifdef LEGACY_ARCH    
