@@ -76,11 +76,6 @@
 #define NV_MAX_SYM_NAME      128
 
 
-#if defined (KERNEL_2_2)
-#define in_irq() (local_irq_count[smp_processor_id()] != 0)
-#endif
-
-
 #ifdef DEBUG
 
 // memory allocation tracking data structs and globals
@@ -1452,9 +1447,9 @@ osMapUserSpace(void *kaddr, void **priv, unsigned int size_bytes, U032 mode)
 
     if ( (size_t) kaddr > VMALLOC_START) {
         nv_ioctl_memory_vtop_t parms;
-        parms.buffer = (NvP64)(NV_UINTPTR_T) kaddr;
+        parms.buffer = kaddr;
         parms.byte_length = 1;
-        parms.physical_addresses = (NvP64)(NV_UINTPTR_T) &paddr;
+        parms.physical_addresses = (unsigned int *) &paddr;
         nv_vtop(&parms, 0);
     } else {
         paddr = (void *) virt_to_phys(kaddr);
@@ -1514,9 +1509,9 @@ osGetKernPhysAddr(void *vaddr)
 
     if ( (size_t) vaddr > VMALLOC_START) {
         nv_ioctl_memory_vtop_t parms;
-        parms.buffer = (NvP64)(NV_UINTPTR_T) vaddr;
+        parms.buffer = vaddr;
         parms.byte_length = 1;
-        parms.physical_addresses = (NvP64)(NV_UINTPTR_T) &ret;
+        parms.physical_addresses = (unsigned int *) &ret;
         nv_vtop(&parms, 0);
     } else {
         ret = virt_to_phys(vaddr);
@@ -2285,10 +2280,8 @@ osRemoveSwapBarrier(int id)
 RM_STATUS
 osSwapBarrier(int id, VOID (*callback)(VOID *), VOID *data)
 {
-    if (sgi_funcs.swap_ready == NULL)
-        return RM_ERROR;
-
-    return sgi_funcs.swap_ready(id, callback, data);
+    (*callback)((VOID *)data);
+    return RM_OK;
 }
 
 
